(function(_window) {
    'use strict';

    /**
     * Regex replacer for HTML escaping.
     *
     * @param {string} match matched character (must exist in HTML_CHARS).
     * @return {string} Html entity
     */
    function htmlReplacer(match) {
        return HTML_CHARS[match];
    }

    var HTML_CHARS = {
        '&': '&amp;',
        '<': '&lt;',
        '>': '&gt;',
        '"': '&quot;',
        "'": '&#x27;',
        '/': '&#x2F;',
        '`': '&#x60;'
    }, utils = {
        /**
         * Return true if the value is an Integer.
         *
         * Uses ES6 function if defined.
         *
         * @param {*} val
         */
        isInteger: Number.isInteger || function(val) {
            return typeof val === "number" &&
                   isFinite(val) &&
                   Math.floor(val) === val;
        },

        /**
         * Functional utility to loop through an array calling a
         * callback each time to test for a condition to be meet, and
         * exit on first find.
         *
         * @param {Array} arr
         * @param {Function} cb
         * @returns {Boolean}
         */
        findFirst: function(arr, cb) {
            var i;

            for(i=0; i < arr.length; i++) {
                if (cb(arr[i])) {
                    return true;
                }
            }

            return false;
        },

        // Port from underscore
        debounce: function(func, wait, immediate) {
            var timeout;
            return function() {
                var context = this, args = arguments;
                var later = function() {
                    timeout = null;
                    if (!immediate) {
                        func.apply(context, args);
                    }
                };
                var callNow = immediate && !timeout;
                clearTimeout(timeout);
                timeout = setTimeout(later, wait);
                if (callNow) {
                    func.apply(context, args);
                }
            };
        },

        /**
         * Escape Html string
         *
         * String escaping meets OWASP recommendations.
         * This method is taken from YUI Escape API.
         * {@see http://yuilibrary.com/yui/docs/api/files/escape_js_escape.js.html}
         *
         * @param {string} html string
         * @return {string} escaped html string
         */
        escapeHtml: function(string) {
            return (string + '').replace(/[&<>"'\/`]/g, htmlReplacer);
        },

        noop: function() { }
    };

    _window.utils = utils;
})(window);
